@@ -1,3 +1,5 @@
+library(lfa)
+
 # need to simulate binomial data
 # these are the dimensions we want
 m <- 300 # this has to be at least this large or `pip` (really `qvalue::pi0est`) fails in stupid ways
@@ -39,17 +41,11 @@ while( m_const ) {
     m_const <- sum( indexes )
 }
 
-# for PCA and other analyses for continuous data, makes sense to centerscale data
-Xc <- t( scale( t( X ) ) )
-
 # and LFs to go with this data
 # since LFA is imported, might as well use it to get actual LFs for our random data
-# to get rid of LFA dependence temporarily, use PCA instead
-kinship_est <- crossprod( Xc ) / m
-# NOTE: get d-1 PCs!
-LF1 <- eigen( kinship_est )$vectors[ , 1L : ( d - 1L ) ]
-# for coherence with LFA, add intercept at the end
-LF1 <- cbind( LF1, 1 )
+LF1 <- lfa( X, d )
+# in practice LF0 is just intercept model
+LF0 <- NULL
 
 # set these parameters (match defaults), which also determine dimensions of null.stat matrix
 # let's use smaller values than defaults so tests are faster
@@ -60,6 +56,9 @@ B <- 2 # default is: round( m * 10 / s )
 # draw continuous (in LFA, drawing bernoulli causes fitting problems I don't particularly want to deal with here)
 covariate <- rnorm( n )
 
+# for PCA and other analyses for continuous data, makes sense to centerscale data
+Xc <- t( scale( t( X ) ) )
+
 test_that( "pip works", {
     # construct some random data
     pvalue <- runif( m )
@@ -73,7 +72,7 @@ test_that( "pip works", {
     expect_error( pip( ) )
     # pass group with wrong length
     expect_error( pip( pvalue = pvalue, group = group[-1], verbose = FALSE ) )
-
+    
     # now successful runs
     expect_silent(
         prob <- pip( pvalue = pvalue, group = group, pi0 = pi0, verbose = FALSE )
@@ -91,23 +90,23 @@ test_that( "pip works", {
     expect_true( all( prob >= 0 ) )
     # expect_true( all( prob <= 0 ) ) # not true!
 
-    ## # a rare error, ultimately in `qvalue::pi0est`, happens in the next line (due to small groups, not enough p-values for good pi0 estimates):
-    ## ## Error (test-jackstraw.R:67:5): pip works
-    ## ## Error: missing or infinite values in inputs are not allowed
-    ## ## Backtrace:
-    ## ##   1. testthat::expect_silent(...) test-jackstraw.R:67:4
-    ## ##   9. jackstraw::pip(pvalue = pvalue, group = group, verbose = FALSE)
-    ## ##  10. qvalue::lfdr(pvalue[group == i], ...) /home/viiia/docs/ochoalab/jackstraw/R/pip.R:45:16
-    ## ##  11. qvalue::pi0est(p, ...)
-    ## ##  12. stats::smooth.spline(lambda, pi0, df = smooth.df)
-    ## # so happens only when there's groups and pi0 has to be estimated
-    ## expect_silent(
-    ##     prob <- pip( pvalue = pvalue, group = group, verbose = FALSE )
-    ## )
-    ## expect_true( is.numeric( prob ) )
-    ## expect_equal( length( prob ), m )
-    ## expect_true( all( prob >= 0 ) )
-    ## # expect_true( all( prob <= 0 ) ) # not true!
+    # a rare error, ultimately in `qvalue::pi0est`, happens in the next line (due to small groups, not enough p-values for good pi0 estimates):
+    ## Error (test-jackstraw.R:67:5): pip works
+    ## Error: missing or infinite values in inputs are not allowed
+    ## Backtrace:
+    ##   1. testthat::expect_silent(...) test-jackstraw.R:67:4
+    ##   9. jackstraw::pip(pvalue = pvalue, group = group, verbose = FALSE)
+    ##  10. qvalue::lfdr(pvalue[group == i], ...) /home/viiia/docs/ochoalab/jackstraw/R/pip.R:45:16
+    ##  11. qvalue::pi0est(p, ...)
+    ##  12. stats::smooth.spline(lambda, pi0, df = smooth.df)
+    # so happens only when there's groups and pi0 has to be estimated
+    expect_silent(
+        prob <- pip( pvalue = pvalue, group = group, verbose = FALSE )
+    )
+    expect_true( is.numeric( prob ) )
+    expect_equal( length( prob ), m )
+    expect_true( all( prob >= 0 ) )
+    # expect_true( all( prob <= 0 ) ) # not true!
 
     expect_silent(
         prob <- pip( pvalue = pvalue, verbose = FALSE )
@@ -155,7 +154,7 @@ test_that( "RSS works" , {
 test_that( "FSTAT works", {
     # LF1 doesn't work as-is because the intercept gets added twice!
     LV <- LF1[ , -d, drop = FALSE ]
-
+    
     # check for missing mandatory data
     # both `dat` and `LV` are required
     expect_error( FSTAT( ) )
@@ -184,7 +183,7 @@ test_that( "FSTAT works", {
         fstat_lm[ i ] <- summary( lm( X[ i, ] ~ LF1 ) )$fstatistic[1]
     }
     expect_equal( obj$fstat, fstat_lm )
-
+    
     # successful run with covariates
     expect_silent(
         obj <- FSTAT( dat = Xc, LV = LV, covariate = covariate )
@@ -201,12 +200,12 @@ test_that( "FSTAT works", {
         obj2 <- FSTAT( dat = Xc, LV = LV, ALV = covariate )
     )
     expect_equal( obj2, obj )
-
+    
     # NOTE: `parametric = TRUE` is not used in this package for any public code, so it's also not tested
-
+    
     #FSTAT(dat, LV, ALV = NULL, covariate = NULL, parametric = FALSE)
 })
-
+    
 
 test_that( "permutationPA works", {
     # data is required
@@ -234,6 +233,59 @@ test_that( "permutationPA works", {
     expect_true( all( obj$p <= 1 ) )
 })
 
+test_that( 'permute_alleles_from_geno works', {
+    # a toy example of extremely structured data, where everybody is homozygous for one or the other allele
+    # (recall n=10 in this toy data)
+    n2 <- n / 2L
+    x <- c( rep.int( 0L, n2 ), rep.int( 2L, n2 ) )
+    # permute data at allele level
+    expect_silent(
+        y <- permute_alleles_from_geno( x )
+    )
+    # we should see that lengths match
+    expect_equal( length( y ), n )
+    # and there are no missing values in this case
+    expect_true( !anyNA( y ) )
+    # confirm that everything is in desired range
+    expect_true( all( y %in% c(0L, 1L, 2L) ) )
+    # for this example, the case without heterozygotes is so rare (0.5^10 = 9.8e-4) let's demand that there be at least one heterozygote
+    expect_true( any( y == 1L ) )
+    # and sum of genotypes equals `n` (in this case true by construction, not true in general)
+    expect_equal( sum( y ), n )
+    
+    # now try on the simulated random data (all rows)
+    # vectorize so number of tests isn't too extreme
+    expect_silent(
+        Y <- t( apply( X, 1L, permute_alleles_from_geno ) )
+    )
+    # perform general tests, still allowing no missingness (not present in simulated data)
+    expect_equal( ncol( Y ), n )
+    expect_true( !anyNA( Y ) )
+    expect_true( all( Y %in% c(0L, 1L, 2L) ) )
+    # returning the same data is practically impossible if permutation is correct
+    expect_true( !all( Y == X ) )
+    # here test that the sums of input and output for each row match
+    expect_equal( rowSums( Y ), rowSums( X ) )
+
+    # we haven't performed tests with missingness in general, and the overall runtime is so high I don't think we want to do it broadly, but here let's just sprinkle random missingness (here there's absolute tolerance for bad things happening, like fixed rows, even rows full of NAs!)
+    X_miss <- X
+    # just a little bit of missingness
+    p_miss <- 0.1
+    # add missing values
+    X_miss[ sample( n*m, n*m*p_miss ) ] <- NA
+    # repeat earlier test!
+    expect_silent(
+        Y_miss <- t( apply( X_miss, 1L, permute_alleles_from_geno ) )
+    )
+    # perform general tests, now allowing missingness!
+    expect_equal( ncol( Y_miss ), n )
+    expect_true( all( Y %in% c(0L, 1L, 2L), na.rm = TRUE ) )
+    # returning the same data is practically impossible if permutation is correct
+    expect_true( !all( Y == X, na.rm = TRUE ) )
+    # here test that the sums of input and output for each row match
+    expect_equal( rowSums( Y, na.rm = TRUE ), rowSums( X, na.rm = TRUE ) )
+})
+
 test_jackstraw_return_val <- function ( obj, s, B, kmeans = FALSE ) {
     # all jackstraw variants return basically the same thing
     # globals used: m, d
@@ -263,7 +315,7 @@ test_jackstraw_return_val <- function ( obj, s, B, kmeans = FALSE ) {
     p <- obj[[ name_p ]]
     expect_true( is.numeric( p ) )
     expect_equal( length( p ), m )
-#    expect_true( !anyNA( p ) ) # in theory there can be NAs, they just don't arise in my simple examples
+    expect_true( !anyNA( p ) ) # in theory there can be NAs, they just don't arise in my simple examples
     expect_true( all( p >= 0, na.rm = TRUE ) )
     expect_true( all( p <= 1, na.rm = TRUE ) )
     # 3) obs.stat
@@ -288,6 +340,123 @@ test_jackstraw_return_val <- function ( obj, s, B, kmeans = FALSE ) {
     }
 }
 
+test_that( "jackstraw_lfa works", {
+    # cause errors due to missing required data
+    # must provide both dat = X and r = d for a minimal successful run
+    expect_error( jackstraw_lfa( ) )
+    expect_error( jackstraw_lfa( dat = X ) )
+    expect_error( jackstraw_lfa( r = d ) )
+    # check that data is matrix
+    expect_error( jackstraw_lfa( dat = 1:10, r = d ) )
+    # pass bad covariates on purpose
+    expect_error( jackstraw_lfa( dat = X, r = d, covariate = 1 ) ) # scalar is bad
+    expect_error( jackstraw_lfa( dat = X, r = d, covariate = covariate[-1] ) ) # length off by 1, vector
+    expect_error( jackstraw_lfa( dat = X, r = d, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
+    expect_error( jackstraw_lfa( dat = X, r = d, covariate = rbind( covariate ) ) ) # transposed matrix
+    
+    # perform a basic run
+
+    # make it silent so we can focus on problem messages
+    expect_silent(
+        obj <- jackstraw_lfa( X, r = d, s = s, B = B, verbose = FALSE )
+    )
+    # check basic jackstraw return object
+    test_jackstraw_return_val( obj, s, B )
+    
+    # test edge cases
+    # s = 1
+    expect_silent(
+        obj <- jackstraw_lfa( X, r = d, s = 1, B = B, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, 1, B )
+    # B = 1
+    expect_silent(
+        obj <- jackstraw_lfa( X, r = d, s = s, B = 1, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, s, 1 )
+    # s = B = 1
+    expect_silent(
+        obj <- jackstraw_lfa( X, r = d, s = 1, B = 1, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, 1, 1 )
+    
+    # this comparison succeeds most of the time, but not 100% of the time
+    # bad fits cause errors randomly, which are rare but over 100 loci it gets less rare
+    # also, things get very slow
+    
+    # test version with covariates
+    expect_silent(
+        obj <- jackstraw_lfa( X, r = d, s = s, B = B, covariate = covariate, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, s, B )
+
+    # test version without default allele-level permutation!
+    expect_silent(
+        obj <- jackstraw_lfa( X, r = d, s = s, B = B, permute_alleles = FALSE, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, s, B )
+
+})
+
+# run alstructure tests only if optional package is available
+if (suppressMessages(suppressWarnings(require(alstructure)))) {
+    
+    test_that("jackstraw_alstructure works", {
+        # define function to pass (uses global `d`)
+        FUN <- function(x) t( alstructure(x, d_hat = d)$Q_hat )
+        
+        # cause errors due to missing required data
+        # must provide all of (dat = X, r = d, FUN = FUN) for a minimal successful run
+        expect_error( jackstraw_alstructure( ) )
+        expect_error( jackstraw_alstructure( dat = X ) )
+        expect_error( jackstraw_alstructure( r = d ) )
+        expect_error( jackstraw_alstructure( FUN = FUN ) )
+        expect_error( jackstraw_alstructure( dat = X, r = d ) )
+        expect_error( jackstraw_alstructure( dat = X, FUN = FUN ) )
+        expect_error( jackstraw_alstructure( r = d, FUN = FUN ) )
+        # check that data is matrix
+        expect_error( jackstraw_alstructure( dat = 1:10, r = d, FUN = FUN ) )
+        # pass bad covariates on purpose
+        expect_error( jackstraw_alstructure( dat = X, r = d, FUN = FUN, covariate = 1 ) ) # scalar is bad
+        expect_error( jackstraw_alstructure( dat = X, r = d, FUN = FUN, covariate = covariate[-1] ) ) # length off by 1, vector
+        expect_error( jackstraw_alstructure( dat = X, r = d, FUN = FUN, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
+        expect_error( jackstraw_alstructure( dat = X, r = d, FUN = FUN, covariate = rbind( covariate ) ) ) # transposed matrix
+        
+        # perform a basic run
+
+        # make it silent so we can focus on problem messages
+        expect_silent(
+            obj <- jackstraw_alstructure( X, r = d, FUN = FUN, s = s, B = B, verbose = FALSE )
+        )
+        # check basic jackstraw return object
+        test_jackstraw_return_val( obj, s, B )
+
+        # test edge cases
+        # s = 1
+        expect_silent(
+            obj <- jackstraw_alstructure( X, r = d, FUN = FUN, s = 1, B = B, verbose = FALSE )
+        )
+        test_jackstraw_return_val( obj, 1, B )
+        # B = 1
+        expect_silent(
+            obj <- jackstraw_alstructure( X, r = d, FUN = FUN, s = s, B = 1, verbose = FALSE )
+        )
+        test_jackstraw_return_val( obj, s, 1 )
+        # s = B = 1
+        expect_silent(
+            obj <- jackstraw_alstructure( X, r = d, FUN = FUN, s = 1, B = 1, verbose = FALSE )
+        )
+        test_jackstraw_return_val( obj, 1, 1 )
+        
+        # test version with covariates
+        expect_silent(
+            obj <- jackstraw_alstructure( X, r = d, FUN = FUN, s = s, B = B, covariate = covariate, verbose = FALSE )
+        )
+        test_jackstraw_return_val( obj, s, B )
+    })
+
+}
+
 test_that("jackstraw_subspace works", {
     FUN <- function( x ) svd( x )$v[ , 1:d, drop = FALSE ]
     # cause errors due to missing required data
@@ -306,7 +475,7 @@ test_that("jackstraw_subspace works", {
     expect_error( jackstraw_subspace( dat = Xc, r = d, FUN = FUN, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_subspace( dat = Xc, r = d, FUN = FUN, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_subspace( dat = Xc, r = d, FUN = FUN, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -332,7 +501,7 @@ test_that("jackstraw_subspace works", {
         obj <- jackstraw_subspace( Xc, r = d, FUN = FUN, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1 )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_subspace( Xc, r = d, FUN = FUN, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -352,7 +521,7 @@ test_that("jackstraw_pca works", {
     expect_error( jackstraw_pca( dat = Xc, r = d, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_pca( dat = Xc, r = d, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_pca( dat = Xc, r = d, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -378,7 +547,7 @@ test_that("jackstraw_pca works", {
         obj <- jackstraw_pca( Xc, r = d, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1 )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_pca( Xc, r = d, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -398,7 +567,7 @@ test_that("jackstraw_rpca works", {
     expect_error( jackstraw_rpca( dat = Xc, r = d, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_rpca( dat = Xc, r = d, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_rpca( dat = Xc, r = d, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -424,7 +593,7 @@ test_that("jackstraw_rpca works", {
         obj <- jackstraw_rpca( Xc, r = d, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1 )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_rpca( Xc, r = d, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -444,7 +613,7 @@ test_that("jackstraw_irlba works", {
     expect_error( jackstraw_irlba( dat = Xc, r = d, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_irlba( dat = Xc, r = d, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_irlba( dat = Xc, r = d, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -470,7 +639,7 @@ test_that("jackstraw_irlba works", {
         obj <- jackstraw_irlba( Xc, r = d, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1 )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_irlba( Xc, r = d, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -487,7 +656,7 @@ test_that( "jackstraw_kmeans works", {
     expect_error( jackstraw_kmeans() )
     expect_error( jackstraw_kmeans( dat = Xc ) )
     expect_error( jackstraw_kmeans( kmeans.dat = kmeans.dat ) )
-
+    
     # check that data is matrix
     expect_error( jackstraw_kmeans( dat = 1:10, kmeans.dat = kmeans.dat ) )
     # pass bad covariates on purpose
@@ -495,7 +664,7 @@ test_that( "jackstraw_kmeans works", {
     expect_error( jackstraw_kmeans( dat = Xc, kmeans.dat = kmeans.dat, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_kmeans( dat = Xc, kmeans.dat = kmeans.dat, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_kmeans( dat = Xc, kmeans.dat = kmeans.dat, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -521,7 +690,7 @@ test_that( "jackstraw_kmeans works", {
         obj <- jackstraw_kmeans( Xc, kmeans.dat = kmeans.dat, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1, kmeans = TRUE )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_kmeans( Xc, kmeans.dat = kmeans.dat, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -538,7 +707,7 @@ test_that( "jackstraw_kmeanspp works", {
     expect_error( jackstraw_kmeanspp() )
     expect_error( jackstraw_kmeanspp( dat = Xc ) )
     expect_error( jackstraw_kmeanspp( kmeans.dat = kmeans.dat ) )
-
+    
     # check that data is matrix
     expect_error( jackstraw_kmeanspp( dat = 1:10, kmeans.dat = kmeans.dat ) )
     # pass bad covariates on purpose
@@ -546,7 +715,7 @@ test_that( "jackstraw_kmeanspp works", {
     expect_error( jackstraw_kmeanspp( dat = Xc, kmeans.dat = kmeans.dat, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_kmeanspp( dat = Xc, kmeans.dat = kmeans.dat, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_kmeanspp( dat = Xc, kmeans.dat = kmeans.dat, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -572,7 +741,7 @@ test_that( "jackstraw_kmeanspp works", {
         obj <- jackstraw_kmeanspp( Xc, kmeans.dat = kmeans.dat, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1, kmeans = TRUE )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_kmeanspp( Xc, kmeans.dat = kmeans.dat, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -584,13 +753,13 @@ test_that( "jackstraw_MiniBatchKmeans works", {
     # a simple k-means run
     batch_size <- 10
     MiniBatchKmeans.output <- ClusterR::MiniBatchKmeans( Xc, clusters = d, batch_size = batch_size )
-
+    
     # cause errors due to missing required data
     # must provide both dat = X and MiniBatchKmeans.output for a minimal successful run
     expect_error( jackstraw_MiniBatchKmeans() )
     expect_error( jackstraw_MiniBatchKmeans( dat = Xc ) )
     expect_error( jackstraw_MiniBatchKmeans( MiniBatchKmeans.output = MiniBatchKmeans.output ) )
-
+    
     # check that data is matrix
     expect_error( jackstraw_MiniBatchKmeans( dat = 1:10, MiniBatchKmeans.output = MiniBatchKmeans.output ) )
     # pass bad covariates on purpose
@@ -598,7 +767,7 @@ test_that( "jackstraw_MiniBatchKmeans works", {
     expect_error( jackstraw_MiniBatchKmeans( dat = Xc, MiniBatchKmeans.output = MiniBatchKmeans.output, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_MiniBatchKmeans( dat = Xc, MiniBatchKmeans.output = MiniBatchKmeans.output, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_MiniBatchKmeans( dat = Xc, MiniBatchKmeans.output = MiniBatchKmeans.output, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -624,7 +793,7 @@ test_that( "jackstraw_MiniBatchKmeans works", {
         obj <- jackstraw_MiniBatchKmeans( Xc, MiniBatchKmeans.output = MiniBatchKmeans.output, batch_size = batch_size, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1, kmeans = TRUE )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_MiniBatchKmeans( Xc, MiniBatchKmeans.output = MiniBatchKmeans.output, batch_size = batch_size, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -635,13 +804,13 @@ test_that( "jackstraw_MiniBatchKmeans works", {
 test_that( "jackstraw_pam works", {
     # a simple PAM run
     pam.dat <- cluster::pam( Xc, k = d )
-
+    
     # cause errors due to missing required data
     # must provide both dat = X and pam.dat for a minimal successful run
     expect_error( jackstraw_pam() )
     expect_error( jackstraw_pam( dat = Xc ) )
     expect_error( jackstraw_pam( pam.dat = pam.dat ) )
-
+    
     # check that data is matrix
     expect_error( jackstraw_pam( dat = 1:10, pam.dat = pam.dat ) )
     # pass bad covariates on purpose
@@ -649,7 +818,7 @@ test_that( "jackstraw_pam works", {
     expect_error( jackstraw_pam( dat = Xc, pam.dat = pam.dat, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_pam( dat = Xc, pam.dat = pam.dat, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_pam( dat = Xc, pam.dat = pam.dat, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -677,7 +846,7 @@ test_that( "jackstraw_pam works", {
     ##     obj <- jackstraw_pam( Xc, pam.dat = pam.dat, s = 1, B = 1, verbose = FALSE )
     ## )
     ## test_jackstraw_return_val( obj, 1, 1, kmeans = TRUE )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_pam( Xc, pam.dat = pam.dat, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -691,7 +860,7 @@ test_that( "jackstraw_cluster works", {
     FUN.dat <- FUN( Xc, centers = d )
     cluster <- FUN.dat$cluster
     centers <- FUN.dat$centers
-
+    
     # cause errors due to missing required data
     # 4 arguments are required for a successful run
     expect_error( jackstraw_cluster() )
@@ -712,7 +881,7 @@ test_that( "jackstraw_cluster works", {
     expect_error( jackstraw_cluster( dat = Xc, cluster = cluster, centers = centers ) )
     expect_error( jackstraw_cluster( dat = Xc, k = d, centers = centers ) )
     expect_error( jackstraw_cluster( dat = Xc, k = d, cluster = cluster ) )
-
+    
     # check that data is matrix
     expect_error( jackstraw_cluster( dat = 1:10, k = d, cluster = cluster, centers = centers ) )
     # check cluster vector length
@@ -725,7 +894,7 @@ test_that( "jackstraw_cluster works", {
     expect_error( jackstraw_cluster( dat = Xc, k = d, cluster = cluster, centers = centers, covariate = covariate[-1] ) ) # length off by 1, vector
     expect_error( jackstraw_cluster( dat = Xc, k = d, cluster = cluster, centers = centers, covariate = cbind( covariate[-1] ) ) ) # length off by 1, matrix
     expect_error( jackstraw_cluster( dat = Xc, k = d, cluster = cluster, centers = centers, covariate = rbind( covariate ) ) ) # transposed matrix
-
+    
     # perform a basic run
 
     # make it silent so we can focus on problem messages
@@ -751,7 +920,7 @@ test_that( "jackstraw_cluster works", {
         obj <- jackstraw_cluster( dat = Xc, k = d, cluster = cluster, centers = centers, algorithm = FUN, s = 1, B = 1, verbose = FALSE )
     )
     test_jackstraw_return_val( obj, 1, 1, kmeans = TRUE )
-
+    
     # test version with covariates
     expect_silent(
         obj <- jackstraw_cluster( dat = Xc, k = d, cluster = cluster, centers = centers, algorithm = FUN, s = s, B = B, covariate = covariate, verbose = FALSE )
@@ -759,6 +928,102 @@ test_that( "jackstraw_cluster works", {
     test_jackstraw_return_val( obj, s, B, kmeans = TRUE )
 })
 
+test_that( "efron_Rsq_snp works", {
+    # data to use
+    xi <- X[1,]
+    pi <- lfa::af_snp(xi, LF1)
+    
+    # cause errors due to missing arguments
+    expect_error( efron_Rsq_snp() )
+    expect_error( efron_Rsq_snp( snp = xi ) )
+    expect_error( efron_Rsq_snp( p1 = pi ) )
+
+    # now a successful run
+    expect_silent(
+        r2 <- efron_Rsq_snp( snp = xi, p1 = pi )
+    )
+    # the basics of what this R^2 should be like
+    expect_true( is.numeric( r2 ) )
+    expect_equal( length( r2 ), 1 )
+    expect_true( !is.na( r2 ) )
+    expect_true( r2 >= 0 )
+    expect_true( r2 <= 1 )
+})
+
+test_that( "efron_Rsq works", {
+    # cause errors due to missing arguments
+    expect_error( efron_Rsq( ) )
+    expect_error( efron_Rsq( X ) )
+    expect_error( efron_Rsq( LF = LF1 ) )
+    # pass non-matrix arguments
+    expect_error( efron_Rsq( as.vector( X ), LF1 ) )
+    expect_error( efron_Rsq( X, as.vector( LF1 ) ) )
+
+    # now a successful run
+    expect_silent(
+        r2 <- efron_Rsq( X, LF1 )
+    )
+    # the basics of what this R^2 should be like
+    expect_true( is.numeric( r2 ) )
+    expect_equal( length( r2 ), m )
+    expect_true( !anyNA( r2 ) )
+    expect_true( all( r2 >= 0 ) )
+    expect_true( all( r2 <= 1 ) )
+})
+
+test_that( "mcfadden_Rsq_snp works", {
+    # LF0 it can't be null here
+    if ( is.null( LF0 ) )
+        LF0 <- matrix(1, n, 1)
+    # data to use
+    xi <- X[1,]
+    p1 <- lfa::af_snp(xi, LF1)
+    p0 <- lfa::af_snp(xi, LF0)
+    
+    # cause errors due to missing arguments
+    # all three arguments are required
+    expect_error( mcfadden_Rsq_snp() )
+    expect_error( mcfadden_Rsq_snp( snp = xi ) )
+    expect_error( mcfadden_Rsq_snp( p1 = p1 ) )
+    expect_error( mcfadden_Rsq_snp( p0 = p0 ) )
+    expect_error( mcfadden_Rsq_snp( snp = xi, p1 = p1 ) )
+    expect_error( mcfadden_Rsq_snp( snp = xi, p0 = p0 ) )
+    expect_error( mcfadden_Rsq_snp( p1 = p1, p0 = p0 ) )
+    
+    # now a successful run
+    expect_silent(
+        r2 <- mcfadden_Rsq_snp( snp = xi, p1 = p1, p0 = p0 )
+    )
+    # the basics of what this R^2 should be like
+    expect_true( is.numeric( r2 ) )
+    expect_equal( length( r2 ), 1 )
+    expect_true( !is.na( r2 ) )
+    expect_true( r2 >= 0 )
+    expect_true( r2 <= 1 )
+})
+
+test_that( "pseudo_Rsq works", {
+    # cause errors due to missing arguments
+    expect_error( pseudo_Rsq( ) )
+    expect_error( pseudo_Rsq( X ) )
+    expect_error( pseudo_Rsq( LF_alt = LF1 ) )
+    # pass non-matrix arguments
+    expect_error( pseudo_Rsq( as.vector( X ), LF1 ) )
+    expect_error( pseudo_Rsq( X, as.vector( LF1 ) ) )
+
+    # now a successful run
+    # LF_null is set to default (intercept only)
+    expect_silent(
+        r2 <- pseudo_Rsq( X, LF1 )
+    )
+    # the basics of what this R^2 should be like
+    expect_true( is.numeric( r2 ) )
+    expect_equal( length( r2 ), m )
+    expect_true( !anyNA( r2 ) )
+    expect_true( all( r2 >= 0 ) )
+    expect_true( all( r2 <= 1 ) )
+})
+
 test_that( "empPvals handles NAs correctly", {
     # `qvalue::empPvals` actually it doesn't, but it's easy to fix with a minor hack, wrapped around internal `empPvals`
     m <- 100
@@ -784,3 +1049,158 @@ test_that( "empPvals handles NAs correctly", {
     expect_true( all( is.na( pvals[ is.na( obs ) ] ) ) )
     expect_true( !anyNA( pvals[ !is.na( obs ) ] ) )
 })
+
+# first write test genotypes somewhere
+file_tmp <- tempfile( 'test-jackstraw', fileext = '.bed' )
+genio::write_bed( file_tmp, X, verbose = FALSE )
+# read it back as a BEDMatrix object
+# read it this way, specifying dimensions, because there's no BIM/FAM files
+X_BM <- BEDMatrix::BEDMatrix( file_tmp, n = n, p = m )
+
+test_that( "jackstraw_BEDMatrix works", {
+    # NOTE: BEDMatrix and genio are both dependencies
+
+    # extra parameters
+    m_chunk <- 1000 # default
+    # recall s==2, let's leave it that way for now
+    random_s <- sample.int( m, s )
+    # need sorted version for some tests
+    random_s_sorted <- sort( random_s )
+    
+    expect_silent(
+        obj <- jackstraw_BEDMatrix( dat = X_BM, random_s, m_chunk = m_chunk )
+    )
+    expect_equal( class( obj ), 'list' )
+    expect_equal( names( obj ), c('dat_full', 'dat_rand', 'file_full', 'file_rand') )
+    expect_true( class( obj$dat_full ) == 'BEDMatrix' )
+    expect_true( class( obj$dat_rand ) == 'BEDMatrix' )
+    expect_equal( class( obj$file_full ), 'character' )
+    expect_equal( class( obj$file_rand ), 'character' )
+    expect_equal( nrow( obj$dat_full ), n )
+    expect_equal( ncol( obj$dat_full ), m )
+    expect_equal( nrow( obj$dat_rand ), n )
+    expect_equal( ncol( obj$dat_rand ), s )
+    # the non-random loci should match the original data
+    # (NOTE: BEDMatrix data has to be transposed and dimnames removed)
+    X_orig_exp <- X[ -random_s, ]
+    X_orig_obs <- t( obj$dat_full[ , -random_s ] )
+    dimnames( X_orig_obs ) <- NULL
+    expect_equal( X_orig_obs, X_orig_exp )
+    # the random data in the full matrix should equal the random matrix
+    # here they're both BEDMatrix so no transposition is required
+    # however, `dat_rand` lists rows in original order, which may disagree with `random_s` (random order), so let's sort here to get matching orders
+    X_rand_exp <- obj$dat_rand[]
+    X_rand_obs <- obj$dat_full[ , random_s_sorted, drop = FALSE ]
+    expect_equal( X_rand_obs, X_rand_exp )
+    
+    # test this edge case
+    m_chunk <- 1
+    expect_silent(
+        obj <- jackstraw_BEDMatrix( dat = X_BM, random_s, m_chunk = m_chunk )
+    )
+    expect_equal( class( obj ), 'list' )
+    expect_equal( names( obj ), c('dat_full', 'dat_rand', 'file_full', 'file_rand') )
+    expect_true( class( obj$dat_full ) == 'BEDMatrix' )
+    expect_true( class( obj$dat_rand ) == 'BEDMatrix' )
+    expect_equal( class( obj$file_full ), 'character' )
+    expect_equal( class( obj$file_rand ), 'character' )
+    expect_equal( nrow( obj$dat_full ), n )
+    expect_equal( ncol( obj$dat_full ), m )
+    expect_equal( nrow( obj$dat_rand ), n )
+    expect_equal( ncol( obj$dat_rand ), s )
+    X_orig_exp <- X[ -random_s, ]
+    X_orig_obs <- t( obj$dat_full[ , -random_s ] )
+    dimnames( X_orig_obs ) <- NULL
+    expect_equal( X_orig_obs, X_orig_exp )
+    X_rand_exp <- obj$dat_rand[]
+    X_rand_obs <- obj$dat_full[ , random_s_sorted, drop = FALSE ]
+    expect_equal( X_rand_obs, X_rand_exp )
+    
+    # test this edge case
+    s <- 1
+    m_chunk <- 1000
+    random_s <- sample.int( m, s )
+    # need sorted version for some tests (not needed if s=1 but meh)
+    random_s_sorted <- sort( random_s )
+    expect_silent(
+        obj <- jackstraw_BEDMatrix( dat = X_BM, random_s, m_chunk = m_chunk )
+    )
+    expect_equal( class( obj ), 'list' )
+    expect_equal( names( obj ), c('dat_full', 'dat_rand', 'file_full', 'file_rand') )
+    expect_true( class( obj$dat_full ) == 'BEDMatrix' )
+    expect_true( class( obj$dat_rand ) == 'BEDMatrix' )
+    expect_equal( class( obj$file_full ), 'character' )
+    expect_equal( class( obj$file_rand ), 'character' )
+    expect_equal( nrow( obj$dat_full ), n )
+    expect_equal( ncol( obj$dat_full ), m )
+    expect_equal( nrow( obj$dat_rand ), n )
+    expect_equal( ncol( obj$dat_rand ), s )
+    X_orig_exp <- X[ -random_s, ]
+    X_orig_obs <- t( obj$dat_full[ , -random_s ] )
+    dimnames( X_orig_obs ) <- NULL
+    expect_equal( X_orig_obs, X_orig_exp )
+    X_rand_exp <- obj$dat_rand[]
+    X_rand_obs <- obj$dat_full[ , random_s_sorted, drop = FALSE ]
+    expect_equal( X_rand_obs, X_rand_exp )
+    
+    # final, double edge case
+    s <- 1
+    m_chunk <- 1
+    expect_silent(
+        obj <- jackstraw_BEDMatrix( dat = X_BM, random_s, m_chunk = m_chunk )
+    )
+    expect_equal( class( obj ), 'list' )
+    expect_equal( names( obj ), c('dat_full', 'dat_rand', 'file_full', 'file_rand') )
+    expect_true( class( obj$dat_full ) == 'BEDMatrix' )
+    expect_true( class( obj$dat_rand ) == 'BEDMatrix' )
+    expect_equal( class( obj$file_full ), 'character' )
+    expect_equal( class( obj$file_rand ), 'character' )
+    expect_equal( nrow( obj$dat_full ), n )
+    expect_equal( ncol( obj$dat_full ), m )
+    expect_equal( nrow( obj$dat_rand ), n )
+    expect_equal( ncol( obj$dat_rand ), s )
+    X_orig_exp <- X[ -random_s, ]
+    X_orig_obs <- t( obj$dat_full[ , -random_s ] )
+    dimnames( X_orig_obs ) <- NULL
+    expect_equal( X_orig_obs, X_orig_exp )
+    X_rand_exp <- obj$dat_rand[]
+    X_rand_obs <- obj$dat_full[ , random_s_sorted, drop = FALSE ]
+    expect_equal( X_rand_obs, X_rand_exp )
+    
+})
+
+test_that( "jackstraw_lfa works with BEDMatrix", {
+    # make it silent so we can focus on problem messages
+    expect_silent(
+        obj <- jackstraw_lfa( X_BM, r = d, s = s, B = B, verbose = FALSE )
+    )
+    # check basic jackstraw return object
+    test_jackstraw_return_val( obj, s, B )
+
+    # test edge cases
+    # s = 1
+    expect_silent(
+        obj <- jackstraw_lfa( X_BM, r = d, s = 1, B = B, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, 1, B )
+    # B = 1
+    expect_silent(
+        obj <- jackstraw_lfa( X_BM, r = d, s = s, B = 1, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, s, 1 )
+    # s = B = 1
+    expect_silent(
+        obj <- jackstraw_lfa( X_BM, r = d, s = 1, B = 1, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, 1, 1 )
+
+    # test version with covariates
+    expect_silent(
+        obj <- jackstraw_lfa( X_BM, r = d, s = s, B = B, covariate = covariate, verbose = FALSE )
+    )
+    test_jackstraw_return_val( obj, s, B )
+})
+
+
+# clean up BEDMatrix example
+invisible( suppressWarnings( file.remove( file_tmp ) ) )
